@@ -1,5 +1,3 @@
-#!/usr/bin/python2
-#
 # parsetree.py, part of Yapps 2 - yet another python parser system
 # Copyright 1999-2003 by Amit J. Patel <amitp@cs.stanford.edu>
 #
@@ -35,12 +33,18 @@ class Generator:
         self.postparser = None
         
         self.tokens = {} # Map from tokens to regexps
-        self.ignore = [] # List of token names to ignore in parsing
+        self.ignore = {} # List of token names to ignore in parsing, map to statements
         self.terminals = [] # List of token names (to maintain ordering)
-        for n, t in tokens:
+        for t in tokens:
+            if len(t) == 3:
+                n,t,s = t
+            else:
+                n,t = t
+                s = None
+
             if n == '#ignore':
                 n = t
-                self.ignore.append(n)
+                self.ignore[n] = s
             if n in self.tokens.keys() and self.tokens[n] != t:
                 print >>sys.stderr, 'Warning: token %s defined more than once.' % n
             self.tokens[n] = t
@@ -199,7 +203,9 @@ class Generator:
         a_set = (repr(a)[1:-1])
         if self.equal_set(a, self.non_ignored_tokens()): a_set = ''
         if self.has_option('context-insensitive-scanner'): a_set = ''
-        return 'self._peek(%s)' % a_set
+        if a_set: a_set += ","
+        
+        return 'self._peek(%s context=_context)' % a_set
     
     def peek_test(self, a, b):
         """Generate a call to test whether the next token (which could be any of
@@ -252,31 +258,39 @@ class Generator:
                 print '    FOLLOW:', ', '.join(top.follow)
                 for x in top.get_children(): queue.append(x)
                 
+    def repr_ignore(self):
+        out="{"
+        for t,s in self.ignore.iteritems():
+            if s is None: s=repr(s)
+            out += "%s:%s," % (repr(t),s)
+        out += "}"
+        return out
+        
     def generate_output(self):
         self.calculate()
         self.write(self.preparser)
         self.write("# Begin -- grammar generated by Yapps\n")
         self.write("import sys, re\n")
-        self.write("import yappsrt\n")
+        self.write("from yapps import runtime\n")
         self.write("\n")
-        self.write("class ", self.name, "Scanner(yappsrt.Scanner):\n")
+        self.write("class ", self.name, "Scanner(runtime.Scanner):\n")
         self.write("    patterns = [\n")
         for p in self.terminals:
             self.write("        (%s, re.compile(%s)),\n" % (
                 repr(p), repr(self.tokens[p])))
         self.write("    ]\n")
-        self.write("    def __init__(self, str):\n")
-        self.write("        yappsrt.Scanner.__init__(self,None,%s,str)\n" %
-                   repr(self.ignore))
+        self.write("    def __init__(self, str,*args,**kw):\n")
+        self.write("        runtime.Scanner.__init__(self,None,%s,str,*args,**kw)\n" %
+                   self.repr_ignore())
         self.write("\n")
         
-        self.write("class ", self.name, "(yappsrt.Parser):\n")
-        self.write(INDENT, "Context = yappsrt.Context\n")
+        self.write("class ", self.name, "(runtime.Parser):\n")
+        self.write(INDENT, "Context = runtime.Context\n")
         for r in self.goals:
             self.write(INDENT, "def ", r, "(self")
             if self.params[r]: self.write(", ", self.params[r])
             self.write(", _parent=None):\n")
-            self.write(INDENT+INDENT, "_context = self.Context(_parent, self._scanner, self._pos, %s, [%s])\n" %
+            self.write(INDENT+INDENT, "_context = self.Context(_parent, self._scanner, %s, [%s])\n" %
                        (repr(r), self.params.get(r, '')))
             self.rules[r].output(self, INDENT+INDENT)
             self.write("\n")
@@ -284,7 +298,7 @@ class Generator:
         self.write("\n")
         self.write("def parse(rule, text):\n")
         self.write("    P = ", self.name, "(", self.name, "Scanner(text))\n")
-        self.write("    return yappsrt.wrap_error_reporter(P, rule)\n")
+        self.write("    return runtime.wrap_error_reporter(P, rule)\n")
         self.write("\n")
         if self.postparser is not None:
             self.write("# End -- grammar generated by Yapps\n")
@@ -355,7 +369,7 @@ class Terminal(Node):
         gen.write(indent)
         if re.match('[a-zA-Z_][a-zA-Z_0-9]*$', self.token):
             gen.write(self.token, " = ")
-        gen.write("self._scan(%s)\n" % repr(self.token))
+        gen.write("self._scan(%s, context=_context)\n" % repr(self.token))
         
 class Eval(Node):
     """This class stores evaluation nodes, from {{ ... }} clauses."""
@@ -547,7 +561,7 @@ class Choice(Node):
 
         if tokens_unseen:
             gen.write(indent, "else:\n")
-            gen.write(indent, INDENT, "raise yappsrt.SyntaxError(_token[0], ")
+            gen.write(indent, INDENT, "raise runtime.SyntaxError(_token[0], ")
             gen.write("'Could not match ", self.rule, "')\n")
         
 class Wrapper(Node):
@@ -586,6 +600,13 @@ class Option(Wrapper):
         gen.write(indent, "if %s:\n" %
                   gen.peek_test(self.first, self.child.first))
         self.child.output(gen, indent+INDENT)
+
+        if gen.has_option('context-insensitive-scanner'):
+            gen.write(indent, "if %s:\n" %
+                    gen.not_peek_test(gen.non_ignored_tokens(), self.follow))
+            gen.write(indent+INDENT, "raise runtime.SyntaxError(pos=self._scanner.get_pos(), context=_context, msg='Need one of ' + ', '.join(%s))\n" %
+                    repr(self.first))
+
         
 class Plus(Wrapper):
     """This class represents a 1-or-more repetition clause of the form A+"""
@@ -613,6 +634,13 @@ class Plus(Wrapper):
         gen.write(indent+INDENT, "if %s: break\n" %
                   gen.not_peek_test(union, self.child.first))
 
+        if gen.has_option('context-insensitive-scanner'):
+            gen.write(indent, "if %s:\n" %
+                    gen.not_peek_test(gen.non_ignored_tokens(), self.follow))
+            gen.write(indent+INDENT, "raise runtime.SyntaxError(pos=self._scanner.get_pos(), context=_context, msg='Need one of ' + ', '.join(%s))\n" %
+                    repr(self.first))
+
+
 class Star(Wrapper):
     """This class represents a 0-or-more repetition clause of the form A*"""
     def setup(self, gen):
@@ -637,9 +665,9 @@ class Star(Wrapper):
         self.child.output(gen, indent+INDENT)
 
         # TODO: need to generate tests like this in lots of rules
-        # TODO: do we need to do this only when it's a context-insensitive scanner?
-        gen.write(indent, "if %s:\n" %
-                  gen.not_peek_test(gen.non_ignored_tokens(), self.follow))
-        gen.write(indent+INDENT, "raise yappsrt.SyntaxError(charpos=self._scanner.get_prev_char_pos(), context=_context, msg='Need one of ' + ', '.join(%s))\n" %
-                  repr(self.first))
+        if gen.has_option('context-insensitive-scanner'):
+            gen.write(indent, "if %s:\n" %
+                    gen.not_peek_test(gen.non_ignored_tokens(), self.follow))
+            gen.write(indent+INDENT, "raise runtime.SyntaxError(pos=self._scanner.get_pos(), context=_context, msg='Need one of ' + ', '.join(%s))\n" %
+                    repr(self.first))
 
